@@ -1,26 +1,31 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
-/**********************************************************************
-
-  rbgobj_type.c -
-
-  $Author: sakai $
-  $Date: 2007/07/16 03:35:53 $
-  created at: Sun Jun  9 20:31:47 JST 2002
-
-  Copyright (C) 2002-2009  Ruby-GNOME2 Project Team
-  Copyright (C) 2002,2003  Masahiro Sakai
-
-**********************************************************************/
+/*
+ *  Copyright (C) 2002-2013  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002,2003  Masahiro Sakai
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA  02110-1301  USA
+ */
 
 #include "rbgprivate.h"
 
+#define RG_TARGET_NAMESPACE rbgobj_cType
+
 /**********************************************************************/
 /* Type Mapping */
 
-#ifdef RBGOBJ_USE_DLPTR
-#include "dl.h"
-#endif
-
 static VALUE rb_cMutex;
 static VALUE lookup_class_mutex;
 
@@ -52,7 +57,6 @@ cinfo_mark(RGObjClassInfo* cinfo)
     rb_gc_mark(cinfo->klass);
 }
 
-
 const RGObjClassInfo *
 rbgobj_lookup_class(VALUE klass)
 {
@@ -77,8 +81,8 @@ rbgobj_lookup_class(VALUE klass)
 }
 
 static const RGObjClassInfo *rbgobj_lookup_class_by_gtype_without_lock(GType gtype,
-								       VALUE parent,
-								       gboolean create_class);
+                                                                       VALUE parent,
+                                                                       gboolean create_class);
 
 static VALUE
 get_superclass(GType gtype)
@@ -95,31 +99,32 @@ get_superclass(GType gtype)
       case G_TYPE_BOXED:
         return rb_cObject;
       case G_TYPE_POINTER:
-#ifdef RBGOBJ_USE_DLPTR
-        return rb_cDLPtrData;
-#else
         return rb_cData;
-#endif
       case G_TYPE_ENUM:
       case G_TYPE_FLAGS:
         return rb_cObject;
       default:
       {
-          const RGObjClassInfo *cinfo_super;
-	  GType parent_type;
+          GType parent_type;
 
-	  parent_type = g_type_parent(gtype);
-	  cinfo_super = rbgobj_lookup_class_by_gtype_without_lock(parent_type,
-								  Qnil,
-								  TRUE);
-          return cinfo_super->klass;
+          parent_type = g_type_parent(gtype);
+          if (parent_type == G_TYPE_INVALID) {
+              return cInstantiatable;
+          } else {
+              const RGObjClassInfo *cinfo_super;
+              cinfo_super =
+                  rbgobj_lookup_class_by_gtype_without_lock(parent_type,
+                                                            Qnil,
+                                                            TRUE);
+              return cinfo_super->klass;
+          }
       }
     }
 }
 
 static const RGObjClassInfo *
 rbgobj_lookup_class_by_gtype_without_lock(GType gtype, VALUE parent,
-					  gboolean create_class)
+                                          gboolean create_class)
 {
     GType fundamental_type;
     RGObjClassInfo* cinfo;
@@ -135,7 +140,7 @@ rbgobj_lookup_class_by_gtype_without_lock(GType gtype, VALUE parent,
         return cinfo;
 
     if (!create_class)
-	return NULL;
+        return NULL;
 
     c = Data_Make_Struct(rb_cData, RGObjClassInfo, cinfo_mark, NULL, cinfo);
     cinfo->gtype = gtype;
@@ -154,11 +159,11 @@ rbgobj_lookup_class_by_gtype_without_lock(GType gtype, VALUE parent,
         if (NIL_P(parent)) parent = get_superclass(gtype);
         cinfo->klass = rb_funcall(rb_cClass, id_new, 1, parent);
         break;
-        
+
     case G_TYPE_INTERFACE:
         cinfo->klass = rb_module_new();
         break;
-        
+
     default:
       if (NIL_P(parent)) parent = get_superclass(gtype);
       if (NIL_P(parent)) {
@@ -173,7 +178,7 @@ rbgobj_lookup_class_by_gtype_without_lock(GType gtype, VALUE parent,
     }
 
     cinfod = (RGObjClassInfoDynamic *)g_hash_table_lookup(dynamic_gtype_list,
-							  g_type_name(gtype));
+                                                          g_type_name(gtype));
     if (cinfod){
         cinfo->mark = cinfod->mark;
         cinfo->free = cinfod->free;
@@ -182,25 +187,25 @@ rbgobj_lookup_class_by_gtype_without_lock(GType gtype, VALUE parent,
 
     rb_hash_aset(klass_to_cinfo, cinfo->klass, c);
     g_hash_table_insert(gtype_to_cinfo, GUINT_TO_POINTER(gtype), cinfo);
-    
+
     if (G_TYPE_IS_CLASSED(gtype))
         gclass = g_type_class_ref(gtype);
-    
+
     if (G_TYPE_IS_INSTANTIATABLE(gtype) || G_TYPE_IS_INTERFACE(gtype))
         rbgobj_define_action_methods(cinfo->klass);
 
     if (G_TYPE_IS_INSTANTIATABLE(gtype)){
         GType* interfaces = NULL;
         guint n_interfaces = 0;
-        int i;
+        guint i;
 
         interfaces = g_type_interfaces(gtype, &n_interfaces);
         for (i = 0; i < n_interfaces; i++){
-	    const RGObjClassInfo *iface_cinfo;
-	    iface_cinfo =
-		rbgobj_lookup_class_by_gtype_without_lock(interfaces[i],
-							  Qnil,
-							  TRUE);
+            const RGObjClassInfo *iface_cinfo;
+            iface_cinfo =
+            rbgobj_lookup_class_by_gtype_without_lock(interfaces[i],
+                                                      Qnil,
+                                                      TRUE);
             rb_include_module(cinfo->klass, iface_cinfo->klass);
         }
         g_free(interfaces);
@@ -239,13 +244,13 @@ rbgobj_lookup_class_by_gtype_body(VALUE data)
     const RGObjClassInfo *cinfo;
 
     cinfo = rbgobj_lookup_class_by_gtype_without_lock(cdata->gtype,
-						      cdata->parent,
-						      cdata->create_class);
+                                                      cdata->parent,
+                                                      cdata->create_class);
     return (VALUE)cinfo;
 }
 
 static VALUE
-rbgobj_lookup_class_by_gtype_ensure(VALUE data)
+rbgobj_lookup_class_by_gtype_ensure(G_GNUC_UNUSED VALUE data)
 {
     rb_funcall(lookup_class_mutex, id_unlock, 0);
     return Qundef;
@@ -259,7 +264,7 @@ rbgobj_lookup_class_by_gtype(GType gtype, VALUE parent)
 
 const RGObjClassInfo *
 rbgobj_lookup_class_by_gtype_full(GType gtype, VALUE parent,
-				  gboolean create_class)
+                                  gboolean create_class)
 {
     RGObjClassByGtypeData data;
 
@@ -268,14 +273,14 @@ rbgobj_lookup_class_by_gtype_full(GType gtype, VALUE parent,
     data.create_class = create_class;
 
     if (create_class) {
-	rb_funcall(lookup_class_mutex, id_lock, 0);
-	return (RGObjClassInfo *)rb_ensure(rbgobj_lookup_class_by_gtype_body,
-					   (VALUE)&data,
-					   rbgobj_lookup_class_by_gtype_ensure,
-					   (VALUE)&data);
+        rb_funcall(lookup_class_mutex, id_lock, 0);
+        return (RGObjClassInfo *)rb_ensure(rbgobj_lookup_class_by_gtype_body,
+                                           (VALUE)&data,
+                                           rbgobj_lookup_class_by_gtype_ensure,
+                                           (VALUE)&data);
     } else {
-	return rbgobj_lookup_class_by_gtype_without_lock(gtype, parent,
-							 create_class);
+        return rbgobj_lookup_class_by_gtype_without_lock(gtype, parent,
+                                                         create_class);
     }
 }
 
@@ -325,16 +330,16 @@ rbgobj_register_class(VALUE klass,
     VALUE c = Qnil;
 
     if (klass2gtype)
-	c = Data_Make_Struct(rb_cData, RGObjClassInfo, cinfo_mark, NULL, cinfo);
+        c = Data_Make_Struct(rb_cData, RGObjClassInfo, cinfo_mark, NULL, cinfo);
     if (gtype2klass && !cinfo)
-	cinfo = g_new(RGObjClassInfo, 1);
+        cinfo = g_new(RGObjClassInfo, 1);
 
     if (cinfo) {
-	cinfo->klass = klass;
-	cinfo->gtype = gtype;
-	cinfo->mark  = NULL;
-	cinfo->free  = NULL;
-	cinfo->flags = 0;
+        cinfo->klass = klass;
+        cinfo->gtype = gtype;
+        cinfo->mark  = NULL;
+        cinfo->free  = NULL;
+        cinfo->flags = 0;
     }
 
     if (klass2gtype)
@@ -351,7 +356,7 @@ rbgobj_register_class(VALUE klass,
     rbgobj_register_class(klass, gtype, FALSE, TRUE)
 
 static void
-Init_typemap()
+init_typemap(void)
 {
     id_new = rb_intern("new");
     id_superclass = rb_intern("superclass");
@@ -391,13 +396,13 @@ Init_typemap()
 /**********************************************************************/
 /* GLib::Type */
 
-VALUE rbgobj_cType;
+VALUE RG_TARGET_NAMESPACE;
 static ID id_gtype;
 
 VALUE
 rbgobj_gtype_new(GType gtype)
 {
-    VALUE result = rb_obj_alloc(rbgobj_cType);
+    VALUE result = rb_obj_alloc(RG_TARGET_NAMESPACE);
     VALUE arg = ULONG2NUM(gtype);
     rb_obj_call_init(result, 1, &arg);
     return result;
@@ -406,7 +411,7 @@ rbgobj_gtype_new(GType gtype)
 GType
 rbgobj_gtype_get(VALUE self)
 {
-    if (RVAL2CBOOL(rb_obj_is_kind_of(self, rbgobj_cType))) {
+    if (RVAL2CBOOL(rb_obj_is_kind_of(self, RG_TARGET_NAMESPACE))) {
         return NUM2ULONG(rb_ivar_get(self, id_gtype));
     } else {
         return CLASS2GTYPE(self);
@@ -415,13 +420,12 @@ rbgobj_gtype_get(VALUE self)
 }
 
 static VALUE
-type_initialize(VALUE self, VALUE type)
+rg_initialize(VALUE self, VALUE type)
 {
     GType gtype;
 
     if (RVAL2CBOOL(rb_obj_is_kind_of(type, rb_cInteger))) {
-        gtype = NUM2UINT(type);
-        // XXX
+        gtype = NUM2ULONG(type);
         if (!g_type_name(gtype))
             gtype = G_TYPE_INVALID;
     } else {
@@ -431,13 +435,13 @@ type_initialize(VALUE self, VALUE type)
     if (G_TYPE_INVALID == gtype)
         rb_raise(rb_eArgError, "invalid type");
 
-    rb_ivar_set(self, id_gtype, UINT2NUM(gtype));
- 
+    rb_ivar_set(self, id_gtype, ULONG2NUM(gtype));
+
     return Qnil;
 }
 
 static VALUE
-type_inspect(VALUE self)
+rg_inspect(VALUE self)
 {
     GType gtype = rbgobj_gtype_get(self);
     gchar* str;
@@ -451,9 +455,9 @@ type_inspect(VALUE self)
 }
 
 static VALUE
-type_compare(VALUE self, VALUE other)
+rg_operator_type_compare(VALUE self, VALUE other)
 {
-    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, RG_TARGET_NAMESPACE)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -471,9 +475,9 @@ type_compare(VALUE self, VALUE other)
 }
 
 static VALUE
-type_eq(VALUE self, VALUE other)
+rg_operator_type_eq(VALUE self, VALUE other)
 {
-    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, RG_TARGET_NAMESPACE)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -483,9 +487,9 @@ type_eq(VALUE self, VALUE other)
 }
 
 static VALUE
-type_lt_eq(VALUE self, VALUE other)
+rg_operator_type_lt_eq(VALUE self, VALUE other)
 {
-    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, RG_TARGET_NAMESPACE)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -495,9 +499,9 @@ type_lt_eq(VALUE self, VALUE other)
 }
 
 static VALUE
-type_gt_eq(VALUE self, VALUE other)
+rg_operator_type_gt_eq(VALUE self, VALUE other)
 {
-    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, RG_TARGET_NAMESPACE)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -507,9 +511,9 @@ type_gt_eq(VALUE self, VALUE other)
 }
 
 static VALUE
-type_lt(VALUE self, VALUE other)
+rg_operator_type_lt(VALUE self, VALUE other)
 {
-    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, RG_TARGET_NAMESPACE)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -519,9 +523,9 @@ type_lt(VALUE self, VALUE other)
 }
 
 static VALUE
-type_gt(VALUE self, VALUE other)
+rg_operator_type_gt(VALUE self, VALUE other)
 {
-    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, rbgobj_cType)))
+    if (!RVAL2CBOOL(rb_obj_is_kind_of(other, RG_TARGET_NAMESPACE)))
         return Qnil;
     else {
         GType a = rbgobj_gtype_get(self);
@@ -531,110 +535,110 @@ type_gt(VALUE self, VALUE other)
 }
 
 static VALUE
-type_to_int(VALUE self)
+rg_hash(VALUE self)
 {
     return rb_ivar_get(self, id_gtype);
 }
 
 static VALUE
-type_to_class(VALUE self)
+rg_to_class(VALUE self)
 {
     return GTYPE2CLASS(rbgobj_gtype_get(self));
 }
 
 static VALUE
-type_fundamental(VALUE self)
+rg_fundamental(VALUE self)
 {
     return rbgobj_gtype_new(G_TYPE_FUNDAMENTAL(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_fundamental(VALUE self)
+rg_fundamental_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_FUNDAMENTAL(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_derived(VALUE self)
+rg_derived_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_DERIVED(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_interface(VALUE self)
+rg_interface_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_INTERFACE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_classed(VALUE self)
+rg_classed_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_CLASSED(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_instantiatable(VALUE self)
+rg_instantiatable_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_INSTANTIATABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_derivable(VALUE self)
+rg_derivable_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_DERIVABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_deep_derivable(VALUE self)
+rg_deep_derivable_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_DEEP_DERIVABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_abstract(VALUE self)
+rg_abstract_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_ABSTRACT(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_value_abstract(VALUE self)
+rg_value_abstract_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_VALUE_ABSTRACT(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_is_value_type(VALUE self)
+rg_value_type_p(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_IS_VALUE_TYPE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_has_value_table(VALUE self)
+rg_has_value_table(VALUE self)
 {
     return CBOOL2RVAL(G_TYPE_HAS_VALUE_TABLE(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_name(VALUE self)
+rg_name(VALUE self)
 {
     return rb_str_new2(g_type_name(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_parent(VALUE self)
+rg_parent(VALUE self)
 {
     GType parent = g_type_parent(rbgobj_gtype_get(self));
     return parent ? rbgobj_gtype_new(parent) : Qnil;
 }
 
 static VALUE
-type_depth(VALUE self)
+rg_depth(VALUE self)
 {
     return UINT2NUM(g_type_depth(rbgobj_gtype_get(self)));
 }
 
 static VALUE
-type_next_base(VALUE leaf_type, VALUE root_type)
+rg_next_base(VALUE leaf_type, VALUE root_type)
 {
     GType ret = g_type_next_base(rbgobj_gtype_get(leaf_type),
                                  rbgobj_gtype_get(root_type));
@@ -642,7 +646,7 @@ type_next_base(VALUE leaf_type, VALUE root_type)
 }
 
 static VALUE
-type_is_a(VALUE self, VALUE is_a_type)
+rg_type_is_a_p(VALUE self, VALUE is_a_type)
 {
     return CBOOL2RVAL(g_type_is_a(rbgobj_gtype_get(self), rbgobj_gtype_get(is_a_type)));
 }
@@ -653,17 +657,17 @@ gpointer              g_type_class_peek              (GType            type);
 void                  g_type_class_unref             (gpointer         g_class);
 gpointer              g_type_class_peek_parent       (gpointer         g_class);
 gpointer              g_type_interface_peek          (gpointer         instance_class,
-						      GType            iface_type);
+                              GType            iface_type);
 gpointer              g_type_interface_peek_parent   (gpointer         g_iface);
 #endif 
 
 static VALUE
-type_children(VALUE self)
+rg_children(VALUE self)
 {
     guint n_children;
     GType* types;
     VALUE result;
-    int i;
+    guint i;
 
     types = g_type_children(rbgobj_gtype_get(self), &n_children);
     result = rb_ary_new2(n_children);
@@ -675,12 +679,12 @@ type_children(VALUE self)
 }
 
 static VALUE
-type_interfaces(VALUE self)
+rg_interfaces(VALUE self)
 {
     guint n_interfaces;
     GType* types;
     VALUE result;
-    int i;
+    guint i;
 
     types = g_type_interfaces(rbgobj_gtype_get(self), &n_interfaces);
     result = rb_ary_new2(n_interfaces);
@@ -692,7 +696,7 @@ type_interfaces(VALUE self)
 }
 
 static VALUE
-type_class_size(VALUE self)
+rg_class_size(VALUE self)
 {
     GTypeQuery query;
     g_type_query(rbgobj_gtype_get(self), &query);
@@ -700,25 +704,33 @@ type_class_size(VALUE self)
 }
 
 static VALUE
-type_instance_size(VALUE self)
+rg_instance_size(VALUE self)
 {
     GTypeQuery query;
     g_type_query(rbgobj_gtype_get(self), &query);
     return UINT2NUM(query.instance_size);
 }
 
-
-static inline
-void _def_fundamental_type(VALUE ary, GType gtype, const char* name)
+static inline void
+_def_fundamental_type(VALUE ary, GType gtype, const char* name)
 {
     VALUE c = rbgobj_gtype_new(gtype);
-    rb_define_const(rbgobj_cType, name, c);
+    rb_define_const(RG_TARGET_NAMESPACE, name, c);
     rb_ary_push(ary, c);
 }
 
-static void
-Init_type()
+/**********************************************************************/
+
+void
+Init_gobject_gtype(void)
 {
+#if !GLIB_CHECK_VERSION(2, 35, 1)
+    g_type_init();
+#endif
+
+    init_typemap();
+
+    /* type */
     rb_cMutex = rb_const_get(rb_cObject, rb_intern("Mutex"));
     id_lock = rb_intern("lock");
     id_unlock = rb_intern("unlock");
@@ -728,50 +740,50 @@ Init_type()
     dynamic_gtype_list = g_hash_table_new(g_str_hash, g_str_equal);
     id_gtype = rb_intern("__gobject_gtype__");
 
-    rbgobj_cType = rb_define_class_under(mGLib, "Type", rb_cObject);
+    RG_TARGET_NAMESPACE = rb_define_class_under(mGLib, "Type", rb_cObject);
 
-    rb_define_alias(CLASS_OF(rbgobj_cType), "[]", "new");
-    rb_define_method(rbgobj_cType, "initialize", type_initialize, 1);
-    rb_define_method(rbgobj_cType, "inspect", type_inspect, 0);
-    rb_define_method(rbgobj_cType, "<=>", type_compare, 1);
-    rb_define_method(rbgobj_cType, "==", type_eq, 1);
-    rb_define_method(rbgobj_cType, "<=", type_lt_eq, 1);
-    rb_define_method(rbgobj_cType, ">=", type_gt_eq, 1);
-    rb_define_method(rbgobj_cType, "<", type_lt, 1);
-    rb_define_method(rbgobj_cType, ">", type_gt, 1);
-    rb_define_method(rbgobj_cType, "eql?", type_eq, 1);
-    rb_define_method(rbgobj_cType, "hash", type_to_int, 0);
-    rb_define_method(rbgobj_cType, "to_i", type_to_int, 0);
-    rb_define_method(rbgobj_cType, "to_int", type_to_int, 0);
-    rb_define_method(rbgobj_cType, "to_class", type_to_class, 0);
+    rb_define_alias(CLASS_OF(RG_TARGET_NAMESPACE), "[]", "new");
+    RG_DEF_METHOD(initialize, 1);
+    RG_DEF_METHOD(inspect, 0);
+    RG_DEF_METHOD_OPERATOR("<=>", type_compare, 1);
+    RG_DEF_METHOD_OPERATOR("==", type_eq, 1);
+    RG_DEF_METHOD_OPERATOR("<=", type_lt_eq, 1);
+    RG_DEF_METHOD_OPERATOR(">=", type_gt_eq, 1);
+    RG_DEF_METHOD_OPERATOR("<", type_lt, 1);
+    RG_DEF_METHOD_OPERATOR(">", type_gt, 1);
+    RG_DEF_ALIAS("eql?", "==");
+    RG_DEF_METHOD(hash, 0);
+    RG_DEF_ALIAS("to_i", "hash");
+    RG_DEF_ALIAS("to_int", "hash");
+    RG_DEF_METHOD(to_class, 0);
 
-    rb_define_method(rbgobj_cType, "fundamental", type_fundamental, 0);
-    rb_define_method(rbgobj_cType, "fundamental?", type_is_fundamental, 0);
-    rb_define_method(rbgobj_cType, "derived?", type_is_derived, 0);
-    rb_define_method(rbgobj_cType, "interface?", type_is_interface, 0);
-    rb_define_method(rbgobj_cType, "classed?", type_is_classed, 0);
-    rb_define_method(rbgobj_cType, "instantiatable?", type_is_instantiatable, 0);
-    rb_define_method(rbgobj_cType, "derivable?", type_is_derivable, 0);
-    rb_define_method(rbgobj_cType, "deep_derivable?", type_is_deep_derivable, 0);
-    rb_define_method(rbgobj_cType, "abstract?", type_is_abstract, 0);
-    rb_define_method(rbgobj_cType, "value_abstract?", type_is_value_abstract, 0);
-    rb_define_method(rbgobj_cType, "value_type?", type_is_value_type, 0);
-    rb_define_method(rbgobj_cType, "has_value_table", type_has_value_table, 0);
+    RG_DEF_METHOD(fundamental, 0);
+    RG_DEF_METHOD_P(fundamental, 0);
+    RG_DEF_METHOD_P(derived, 0);
+    RG_DEF_METHOD_P(interface, 0);
+    RG_DEF_METHOD_P(classed, 0);
+    RG_DEF_METHOD_P(instantiatable, 0);
+    RG_DEF_METHOD_P(derivable, 0);
+    RG_DEF_METHOD_P(deep_derivable, 0);
+    RG_DEF_METHOD_P(abstract, 0);
+    RG_DEF_METHOD_P(value_abstract, 0);
+    RG_DEF_METHOD_P(value_type, 0);
+    RG_DEF_METHOD(has_value_table, 0);
 
-    rb_define_method(rbgobj_cType, "name", type_name, 0);
-    rb_define_method(rbgobj_cType, "to_s", type_name, 0);
-    rb_define_method(rbgobj_cType, "parent", type_parent, 0);
-    rb_define_method(rbgobj_cType, "depth", type_depth, 0);
-    rb_define_method(rbgobj_cType, "next_base", type_next_base, 1);
-    rb_define_method(rbgobj_cType, "type_is_a?", type_is_a, 1);
-    rb_define_method(rbgobj_cType, "children", type_children, 0);
-    rb_define_method(rbgobj_cType, "interfaces", type_interfaces, 0);
-    rb_define_method(rbgobj_cType, "class_size", type_class_size, 0);
-    rb_define_method(rbgobj_cType, "instance_size", type_instance_size, 0);
+    RG_DEF_METHOD(name, 0);
+    RG_DEF_ALIAS("to_s", "name");
+    RG_DEF_METHOD(parent, 0);
+    RG_DEF_METHOD(depth, 0);
+    RG_DEF_METHOD(next_base, 1);
+    RG_DEF_METHOD_P(type_is_a, 1);
+    RG_DEF_METHOD(children, 0);
+    RG_DEF_METHOD(interfaces, 0);
+    RG_DEF_METHOD(class_size, 0);
+    RG_DEF_METHOD(instance_size, 0);
 
     {
     VALUE ary = rb_ary_new();
-    rb_define_const(rbgobj_cType, "FUNDAMENTAL_MAX", INT2FIX(G_TYPE_FUNDAMENTAL_MAX));
+    rb_define_const(RG_TARGET_NAMESPACE, "FUNDAMENTAL_MAX", INT2FIX(G_TYPE_FUNDAMENTAL_MAX));
     _def_fundamental_type(ary, G_TYPE_NONE,      "NONE");
     _def_fundamental_type(ary, G_TYPE_INTERFACE, "INTERFACE");
     _def_fundamental_type(ary, G_TYPE_CHAR,      "CHAR");
@@ -792,15 +804,6 @@ Init_type()
     _def_fundamental_type(ary, G_TYPE_BOXED,     "BOXED");
     _def_fundamental_type(ary, G_TYPE_PARAM,     "PARAM");
     _def_fundamental_type(ary, G_TYPE_OBJECT,    "OBJECT");
-    rb_define_const(rbgobj_cType, "FUNDAMENTAL_TYPES", ary); /* FIXME: better name */
+    rb_define_const(RG_TARGET_NAMESPACE, "FUNDAMENTAL_TYPES", ary); /* FIXME: better name */
     }
 }
-
-/**********************************************************************/
-
-void Init_gobject_gtype()
-{
-    g_type_init();
-    Init_typemap();
-    Init_type();
-}
