@@ -2,7 +2,7 @@
 #
 # Author:: Nathaniel Talbott.
 # Copyright:: Copyright (c) 2000-2002 Nathaniel Talbott. All rights reserved.
-#             Copyright (c) 2009-2013 Kouhei Sutou. All rights reserved.
+#             Copyright (c) 2009-2014 Kouhei Sutou. All rights reserved.
 # License:: Ruby license.
 
 require 'test/unit'
@@ -103,13 +103,7 @@ module Test
       end
 
       def inspect_tag(tag)
-        begin
-          throw tag
-        rescue NameError
-          tag.to_s.inspect
-        rescue ArgumentError
-          tag.inspect
-        end
+        tag.inspect
       end
 
       def add_failure(message, location=caller, options=nil)
@@ -148,15 +142,24 @@ module Test
         }
       end
 
-      def test_assert_equal
-        check_nothing_fails {
-          assert_equal("string1", "string1")
-        }
-        check_nothing_fails {
-          assert_equal("string1", "string1", "successful assert_equal")
-        }
+      class TestAssertEqual < self
+        class TestSuccess < self
+          def test_without_message
+            check_nothing_fails {
+              assert_equal("string1", "string1")
+            }
+          end
 
-        message = <<-EOM.chomp
+          def test_with_message
+            check_nothing_fails {
+              assert_equal("string1", "string1", "successful assert_equal")
+            }
+          end
+        end
+
+        class TestFailure < self
+          def test_without_message
+            message = <<-EOM.chomp
 <"string1"> expected but was
 <"string2">.
 
@@ -166,11 +169,13 @@ diff:
 + string2
 ?       ^
 EOM
-        check_fail(message) {
-          assert_equal("string1", "string2")
-        }
+            check_fail(message) {
+              assert_equal("string1", "string2")
+            }
+          end
 
-        message = <<-EOM.chomp
+          def test_with_message
+            message = <<-EOM.chomp
 failed assert_equal.
 <"string1"> expected but was
 <"string2">.
@@ -181,11 +186,15 @@ diff:
 + string2
 ?       ^
 EOM
-        check_fail(message) {
-          assert_equal("string1", "string2", "failed assert_equal")
-        }
+           check_fail(message) {
+             assert_equal("string1", "string2", "failed assert_equal")
+            }
+          end
+        end
 
-        message = <<-EOM.chomp
+        class TestSystemMessage < self
+          def test_different_type
+            message = <<-EOM.chomp
 <"111111"> expected but was
 <111111>.
 
@@ -194,31 +203,31 @@ diff:
 ? -      -
 + 111111
 EOM
-        check_fail(message) do
-          assert_equal("111111", 111111)
-        end
-      end
+            check_fail(message) do
+              assert_equal("111111", 111111)
+            end
+          end
 
-      def test_assert_equal_with_long_line
-        expected = ["0123456789",
-                    "1123456789",
-                    "2123456789",
-                    "3123456789",
-                    "4123456789",
-                    "5123456789",
-                    "6123456789",
-                    "7123456789",
-                    "8123456789"].join
-        actual =   ["0000000000",
-                    "1123456789",
-                    "2123456789",
-                    "3123456789",
-                    "4123456789",
-                    "5123456789",
-                    "6123456789",
-                    "7123456789",
-                    "8123456789"].join
-        message = <<-EOM.chomp
+          def test_long_line
+            expected = ["0123456789",
+                        "1123456789",
+                        "2123456789",
+                        "3123456789",
+                        "4123456789",
+                        "5123456789",
+                        "6123456789",
+                        "7123456789",
+                        "8123456789"].join
+            actual =   ["0000000000",
+                        "1123456789",
+                        "2123456789",
+                        "3123456789",
+                        "4123456789",
+                        "5123456789",
+                        "6123456789",
+                        "7123456789",
+                        "8123456789"].join
+            message = <<-EOM.chomp
 <"#{expected}"> expected but was
 <"#{actual}">.
 
@@ -235,35 +244,35 @@ folded diff:
 ?  ^^^^^^^^^
   898123456789
 EOM
-        check_fail(message) do
-          assert_equal(expected, actual)
-        end
-      end
+            check_fail(message) do
+              assert_equal(expected, actual)
+            end
+          end
 
-      def test_assert_equal_for_too_small_difference
-        message = <<-EOM.chomp
+          def test_too_small_difference
+            message = <<-EOM.chomp
 <1> expected but was
 <2>.
 EOM
-        check_fail(message) do
-          assert_equal(1, 2)
-        end
-      end
+            check_fail(message) do
+              assert_equal(1, 2)
+            end
+          end
 
-      def test_assert_equal_for_same_inspected_objects
-        now = Time.now
-        now_without_usec = Time.at(now.to_i)
-        message = <<-EOM.chomp
+          def test_same_inspected_objects
+            now = Time.now
+            now_without_usec = Time.at(now.to_i)
+            message = <<-EOM.chomp
 <#{now.inspect}> expected but was
 <#{now.inspect}>.
 EOM
-        check_fail(message) do
-          assert_equal(now, now_without_usec)
-        end
-      end
+            check_fail(message) do
+              assert_equal(now, now_without_usec)
+            end
+          end
 
-      def test_assert_equal_with_multi_lines_result
-        message = <<-EOM.chomp
+          def test_multi_lines_result
+            message = <<-EOM.chomp
 <#{"a\nb".inspect}> expected but was
 <#{"x".inspect}>.
 
@@ -272,13 +281,13 @@ diff:
 - a
 - b
 EOM
-        check_fail(message) do
-          assert_equal("a\nb", "x")
-        end
-      end
+            check_fail(message) do
+              assert_equal("a\nb", "x")
+            end
+          end
 
-      def test_assert_equal_with_large_string
-        message = <<-EOM.chomp
+          def test_large_string
+            message = <<-EOM.chomp
 <#{("a\n" + "x" * 997).inspect}> expected but was
 <#{"x".inspect}>.
 
@@ -293,25 +302,25 @@ folded diff:
 #{(["- " + ("x" * 78)] * 12).join("\n")}
 - #{"x" * 61}
 EOM
-        check_fail(message) do
-          assert_equal("a\n" + "x" * 997, "x")
-        end
+            check_fail(message) do
+              assert_equal("a\n" + "x" * 997, "x")
+            end
 
-        message = <<-EOM.chomp
+            message = <<-EOM.chomp
 <#{("a\n" + "x" * 998).inspect}> expected but was
 <#{"x".inspect}>.
 EOM
-        check_fail(message) do
-          assert_equal("a\n" + "x" * 998, "x")
-        end
-      end
+            check_fail(message) do
+              assert_equal("a\n" + "x" * 998, "x")
+            end
+          end
 
-      def test_assert_equal_with_max_diff_target_string_size
-        key = "TEST_UNIT_MAX_DIFF_TARGET_STRING_SIZE"
-        before_value = ENV[key]
-        ENV[key] = "100"
-        begin
-          message = <<-EOM.chomp
+          def test_max_diff_target_string_size
+            key = "TEST_UNIT_MAX_DIFF_TARGET_STRING_SIZE"
+            before_value = ENV[key]
+            ENV[key] = "100"
+            begin
+              message = <<-EOM.chomp
 <#{("a\n" + "x" * 97).inspect}> expected but was
 <#{"x".inspect}>.
 
@@ -326,61 +335,61 @@ folded diff:
 #{(["- " + ("x" * 78)]).join("\n")}
 - #{"x" * 19}
 EOM
-          check_fail(message) do
-            assert_equal("a\n" + "x" * 97, "x")
-          end
+              check_fail(message) do
+                assert_equal("a\n" + "x" * 97, "x")
+              end
 
-          message = <<-EOM.chomp
+              message = <<-EOM.chomp
 <#{("a\n" + "x" * 98).inspect}> expected but was
 <#{"x".inspect}>.
 EOM
-          check_fail(message) do
-            assert_equal("a\n" + "x" * 98, "x")
+              check_fail(message) do
+                assert_equal("a\n" + "x" * 98, "x")
+              end
+            ensure
+              ENV[key] = before_value
+            end
           end
-        ensure
-          ENV[key] = before_value
-        end
-      end
 
-      def test_assert_equal_with_different_encoding
-        utf8_string = "こんにちは"
-        unless utf8_string.respond_to?(:force_encoding)
-          omit("encoding test is for Ruby >= 1.9")
-        end
-        ascii_8bit_string = utf8_string.dup.force_encoding("ascii-8bit")
-        message = <<-EOM.chomp
+          def test_different_encoding
+            utf8_string = "こんにちは"
+            unless utf8_string.respond_to?(:force_encoding)
+              omit("encoding test is for Ruby >= 1.9")
+            end
+            ascii_8bit_string = utf8_string.dup.force_encoding("ascii-8bit")
+            message = <<-EOM.chomp
 <#{utf8_string.inspect}>("UTF-8") expected but was
 <#{ascii_8bit_string.inspect}>("ASCII-8BIT").
 EOM
-        check_fail(message) do
-          assert_equal(utf8_string, ascii_8bit_string)
-        end
-      end
+            check_fail(message) do
+              assert_equal(utf8_string, ascii_8bit_string)
+            end
+          end
 
-      def test_assert_equal_with_different_hash
-        designers = {
-          "Ruby" => "Matz",
-          "Lisp" => "John McCarthy",
-        }
-        categories = {
-          "LL" => ["Ruby", "Python"],
-          "Heavy" => ["C", "C++"],
-        }
-        message = <<-EOM.chomp
+          def test_different_hash
+            designers = {
+              "Ruby" => "Matz",
+              "Lisp" => "John McCarthy",
+            }
+            categories = {
+              "LL" => ["Ruby", "Python"],
+              "Heavy" => ["C", "C++"],
+            }
+            message = <<-EOM.chomp
 <{"Lisp"=>"John McCarthy", "Ruby"=>"Matz"}> expected but was
 <{"Heavy"=>["C", "C++"], "LL"=>["Ruby", "Python"]}>.
 EOM
-        check_fail(message) do
-          assert_equal(designers, categories)
-        end
-      end
+            check_fail(message) do
+              assert_equal(designers, categories)
+            end
+          end
 
-      def test_assert_equal_with_recursive_hash
-        alice = {"name" => "Alice"}
-        bob = {"name" => "Bob"}
-        alice["followers"] = [bob]
-        bob["followers"] = [alice]
-        message = <<-EOM.chomp
+          def test_recursive_hash
+            alice = {"name" => "Alice"}
+            bob = {"name" => "Bob"}
+            alice["followers"] = [bob]
+            bob["followers"] = [alice]
+            message = <<-EOM.chomp
 <{"followers"=>[{"followers"=>[{...}], "name"=>"Bob"}], "name"=>"Alice"}> expected but was
 <{"followers"=>[{"followers"=>[{...}], "name"=>"Alice"}], "name"=>"Bob"}>.
 
@@ -390,8 +399,31 @@ diff:
 + {"followers"=>[{"followers"=>[{...}], "name"=>"Alice"}], "name"=>"Bob"}
 ?                                                       +++++++++++++++++
 EOM
-        check_fail(message) do
-          assert_equal(alice, bob)
+            check_fail(message) do
+              assert_equal(alice, bob)
+            end
+          end
+
+          def test_numeric
+            numeric_family_class = Class.new(Numeric) do
+              def inspect
+                "inspect is called"
+              end
+
+              def to_s
+                "to_s is called"
+              end
+            end
+            numeric = numeric_family_class.new
+
+            message = <<-MESSAGE.chomp
+<to_s is called> expected but was
+<"must be failed">.
+            MESSAGE
+            check_fail(message) do
+              assert_equal(numeric, "must be failed")
+            end
+          end
         end
       end
 
