@@ -122,6 +122,9 @@ typedef enum
     MILTER_MACRO_STAGE_END_OF_HEADER = 6
 } MilterMacroStage;
 
+gint       milter_status_compare        (MilterStatus status1,
+                                         MilterStatus status2);
+
 G_END_DECLS
 
 #endif /* __MILTER_PROTOCOL_H__ */
