@@ -55,12 +55,14 @@ static struct sigaction default_sigabort_action;
 static struct sigaction default_sigint_action;
 static struct sigaction default_sigterm_action;
 static struct sigaction default_sighup_action;
+static struct sigaction default_sigusr1_action;
 
 static gboolean set_sigsegv_action = TRUE;
 static gboolean set_sigabort_action = TRUE;
 static gboolean set_sigint_action = TRUE;
 static gboolean set_sigterm_action = TRUE;
 static gboolean set_sighup_action = TRUE;
+static gboolean set_sigusr1_action = TRUE;
 
 #define milter_manager_error(...) G_STMT_START  \
 {                                               \
@@ -229,6 +231,12 @@ reload_configuration_request (int signum)
     }
 }
 
+static void
+reopen_log (int signum)
+{
+    milter_logger_reopen(milter_logger());
+}
+
 static void
 cb_error (MilterErrorEmittable *emittable, GError *error, gpointer user_data)
 {
@@ -563,65 +571,6 @@ load_configuration (MilterManager *manager)
     }
 }
 
-static gchar report_stack_trace_window[4096];
-static void
-report_stack_trace (int signum)
-{
-    int fds[2];
-    int in_fd;
-    const char *label = "unknown";
-    int original_stdout_fileno;
-    gssize i, last_new_line, size;
-
-    switch (signum) {
-    case SIGSEGV:
-        label = "SEGV";
-        if (sigaction(SIGSEGV, &default_sigsegv_action, NULL) != -1)
-            set_sigsegv_action = FALSE;
-        break;
-    case SIGABRT:
-        label = "ABORT";
-        if (sigaction(SIGABRT, &default_sigabort_action, NULL) != -1)
-            set_sigabort_action = FALSE;
-        break;
-    }
-
-    if (pipe(fds) == -1) {
-        milter_critical("[%s] unable to open pipe for collecting stack trace",
-                        label);
-        return;
-    }
-
-    original_stdout_fileno = dup(STDOUT_FILENO);
-    dup2(fds[1], STDOUT_FILENO);
-    g_on_error_stack_trace(g_get_prgname());
-    dup2(original_stdout_fileno, STDOUT_FILENO);
-    close(original_stdout_fileno);
-    close(fds[1]);
-
-    in_fd = fds[0];
-    while ((size = read(in_fd,
-                        report_stack_trace_window,
-                        sizeof(report_stack_trace_window))) > 0) {
-        last_new_line = 0;
-        for (i = 0; i < size; i++) {
-            if (report_stack_trace_window[i] == '\n') {
-                report_stack_trace_window[i] = '\0';
-                milter_critical("%s", report_stack_trace_window + last_new_line);
-                last_new_line = i + 1;
-            }
-        }
-        if (last_new_line < size) {
-            report_stack_trace_window[size] = '\0';
-            milter_critical("%s", report_stack_trace_window + last_new_line);
-        }
-    }
-
-    close(fds[0]);
-
-    shutdown_client(SIGINT);
-}
-
 gboolean
 milter_manager_main (void)
 {
@@ -632,9 +581,9 @@ milter_manager_main (void)
     MilterEventLoop *loop;
     gboolean remove_socket, daemon;
     GError *error = NULL;
-    struct sigaction report_stack_trace_action;
     struct sigaction shutdown_client_action;
     struct sigaction reload_configuration_request_action;
+    struct sigaction reopen_log_action;
 
     manager = the_manager;
     config = milter_manager_get_configuration(manager);
@@ -726,9 +675,9 @@ milter_manager_main (void)
     sigemptyset(&handler ## _action.sa_mask);   \
     handler ## _action.sa_flags = 0
 
-    SETUP_SIGNAL_ACTION(report_stack_trace);
     SETUP_SIGNAL_ACTION(shutdown_client);
     SETUP_SIGNAL_ACTION(reload_configuration_request);
+    SETUP_SIGNAL_ACTION(reopen_log);
 #undef SETUP_SIGNAL_ACTION
 
 #define SET_SIGNAL_ACTION(SIGNAL, signal, action)               \
@@ -737,11 +686,10 @@ milter_manager_main (void)
                   &default_sig ## signal ## _action) == -1)     \
         set_sig ## signal ## _action = FALSE
 
-    SET_SIGNAL_ACTION(SEGV, segv, report_stack_trace_action);
-    SET_SIGNAL_ACTION(ABRT, abort, report_stack_trace_action);
     SET_SIGNAL_ACTION(INT, int, shutdown_client_action);
     SET_SIGNAL_ACTION(TERM, term, shutdown_client_action);
     SET_SIGNAL_ACTION(HUP, hup, reload_configuration_request_action);
+    SET_SIGNAL_ACTION(USR1, usr1, reopen_log_action);
 #undef SET_SIGNAL_ACTION
 
     if (!milter_client_run(client, &error)) {
@@ -759,6 +707,7 @@ milter_manager_main (void)
     UNSET_SIGNAL_ACTION(INT, int);
     UNSET_SIGNAL_ACTION(TERM, term);
     UNSET_SIGNAL_ACTION(HUP, hup);
+    UNSET_SIGNAL_ACTION(USR1, usr1);
 #undef UNSET_SIGNAL_ACTION
 
     if (controller)
