@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -24,6 +24,22 @@
 #include <milter/server/milter-server-context.h>
 #include <milter/server/milter-server-enum-types.h>
 
+/**
+ * milter_server_init:
+ *
+ * Call this function before using any other the milter-server
+ * library functions.
+ */
+void milter_server_init (void);
+
+/**
+ * milter_quit:
+ *
+ * Call this function after the milter-server library use.
+ */
+void milter_server_quit (void);
+
+
 #endif /* __MILTER_SERVER_H__ */
 
 /*
